# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from scaleway_core.bridge import (
    unmarshal_Money,
)
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from .types import (
    ScalewayLb,
    ScalewayLbBackendConfig,
    ScalewayS3BackendConfig,
    BackendStage,
    CacheStage,
    DNSStage,
    PipelineError,
    Pipeline,
    TLSSecret,
    TLSStage,
    PipelineStages,
    PurgeRequest,
    CheckDomainResponse,
    CheckLbOriginResponse,
    CheckPEMChainResponse,
    PlanDetails,
    PlanUsageDetails,
    GetBillingResponse,
    ListBackendStagesResponse,
    ListCacheStagesResponse,
    ListDNSStagesResponse,
    ListPipelinesResponse,
    ListPipelinesWithStagesResponse,
    ListPlansResponse,
    ListPurgeRequestsResponse,
    ListTLSStagesResponse,
    Plan,
    CheckDomainRequest,
    CheckLbOriginRequest,
    CheckPEMChainRequestSecretChain,
    CheckPEMChainRequest,
    CreateBackendStageRequest,
    CreateCacheStageRequest,
    CreateDNSStageRequest,
    CreatePipelineRequest,
    CreatePurgeRequestRequest,
    CreateTLSStageRequest,
    SelectPlanRequest,
    UpdateBackendStageRequest,
    UpdateCacheStageRequest,
    UpdateDNSStageRequest,
    UpdatePipelineRequest,
    TLSSecretsConfig,
    UpdateTLSStageRequest,
)


def unmarshal_ScalewayLb(data: Any) -> ScalewayLb:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ScalewayLb' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("zone", None)
    if field is not None:
        args["zone"] = field

    field = data.get("frontend_id", None)
    if field is not None:
        args["frontend_id"] = field

    field = data.get("is_ssl", None)
    if field is not None:
        args["is_ssl"] = field
    else:
        args["is_ssl"] = None

    field = data.get("domain_name", None)
    if field is not None:
        args["domain_name"] = field
    else:
        args["domain_name"] = None

    return ScalewayLb(**args)


def unmarshal_ScalewayLbBackendConfig(data: Any) -> ScalewayLbBackendConfig:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ScalewayLbBackendConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("lbs", None)
    if field is not None:
        args["lbs"] = (
            [unmarshal_ScalewayLb(v) for v in field] if field is not None else None
        )

    return ScalewayLbBackendConfig(**args)


def unmarshal_ScalewayS3BackendConfig(data: Any) -> ScalewayS3BackendConfig:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ScalewayS3BackendConfig' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("bucket_name", None)
    if field is not None:
        args["bucket_name"] = field
    else:
        args["bucket_name"] = None

    field = data.get("bucket_region", None)
    if field is not None:
        args["bucket_region"] = field
    else:
        args["bucket_region"] = None

    field = data.get("is_website", None)
    if field is not None:
        args["is_website"] = field
    else:
        args["is_website"] = None

    return ScalewayS3BackendConfig(**args)


def unmarshal_BackendStage(data: Any) -> BackendStage:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'BackendStage' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("pipeline_id", None)
    if field is not None:
        args["pipeline_id"] = field
    else:
        args["pipeline_id"] = None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("scaleway_s3", None)
    if field is not None:
        args["scaleway_s3"] = unmarshal_ScalewayS3BackendConfig(field)
    else:
        args["scaleway_s3"] = None

    field = data.get("scaleway_lb", None)
    if field is not None:
        args["scaleway_lb"] = unmarshal_ScalewayLbBackendConfig(field)
    else:
        args["scaleway_lb"] = None

    return BackendStage(**args)


def unmarshal_CacheStage(data: Any) -> CacheStage:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'CacheStage' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("pipeline_id", None)
    if field is not None:
        args["pipeline_id"] = field
    else:
        args["pipeline_id"] = None

    field = data.get("fallback_ttl", None)
    if field is not None:
        args["fallback_ttl"] = field
    else:
        args["fallback_ttl"] = None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("backend_stage_id", None)
    if field is not None:
        args["backend_stage_id"] = field
    else:
        args["backend_stage_id"] = None

    return CacheStage(**args)


def unmarshal_DNSStage(data: Any) -> DNSStage:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'DNSStage' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("fqdns", None)
    if field is not None:
        args["fqdns"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("pipeline_id", None)
    if field is not None:
        args["pipeline_id"] = field
    else:
        args["pipeline_id"] = None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("tls_stage_id", None)
    if field is not None:
        args["tls_stage_id"] = field
    else:
        args["tls_stage_id"] = None

    field = data.get("cache_stage_id", None)
    if field is not None:
        args["cache_stage_id"] = field
    else:
        args["cache_stage_id"] = None

    field = data.get("backend_stage_id", None)
    if field is not None:
        args["backend_stage_id"] = field
    else:
        args["backend_stage_id"] = None

    return DNSStage(**args)


def unmarshal_PipelineError(data: Any) -> PipelineError:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'PipelineError' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("stage", None)
    if field is not None:
        args["stage"] = field

    field = data.get("code", None)
    if field is not None:
        args["code"] = field

    field = data.get("severity", None)
    if field is not None:
        args["severity"] = field

    field = data.get("message", None)
    if field is not None:
        args["message"] = field

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    return PipelineError(**args)


def unmarshal_Pipeline(data: Any) -> Pipeline:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Pipeline' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("name", None)
    if field is not None:
        args["name"] = field

    field = data.get("description", None)
    if field is not None:
        args["description"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("errors", None)
    if field is not None:
        args["errors"] = (
            [unmarshal_PipelineError(v) for v in field] if field is not None else None
        )

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("organization_id", None)
    if field is not None:
        args["organization_id"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("dns_stage_id", None)
    if field is not None:
        args["dns_stage_id"] = field
    else:
        args["dns_stage_id"] = None

    return Pipeline(**args)


def unmarshal_TLSSecret(data: Any) -> TLSSecret:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'TLSSecret' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("secret_id", None)
    if field is not None:
        args["secret_id"] = field

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    return TLSSecret(**args)


def unmarshal_TLSStage(data: Any) -> TLSStage:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'TLSStage' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("secrets", None)
    if field is not None:
        args["secrets"] = (
            [unmarshal_TLSSecret(v) for v in field] if field is not None else None
        )

    field = data.get("managed_certificate", None)
    if field is not None:
        args["managed_certificate"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("pipeline_id", None)
    if field is not None:
        args["pipeline_id"] = field
    else:
        args["pipeline_id"] = None

    field = data.get("certificate_expires_at", None)
    if field is not None:
        args["certificate_expires_at"] = (
            parser.isoparse(field) if isinstance(field, str) else field
        )
    else:
        args["certificate_expires_at"] = None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("cache_stage_id", None)
    if field is not None:
        args["cache_stage_id"] = field
    else:
        args["cache_stage_id"] = None

    field = data.get("backend_stage_id", None)
    if field is not None:
        args["backend_stage_id"] = field
    else:
        args["backend_stage_id"] = None

    return TLSStage(**args)


def unmarshal_PipelineStages(data: Any) -> PipelineStages:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'PipelineStages' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("dns_stages", None)
    if field is not None:
        args["dns_stages"] = (
            [unmarshal_DNSStage(v) for v in field] if field is not None else None
        )

    field = data.get("tls_stages", None)
    if field is not None:
        args["tls_stages"] = (
            [unmarshal_TLSStage(v) for v in field] if field is not None else None
        )

    field = data.get("cache_stages", None)
    if field is not None:
        args["cache_stages"] = (
            [unmarshal_CacheStage(v) for v in field] if field is not None else None
        )

    field = data.get("backend_stages", None)
    if field is not None:
        args["backend_stages"] = (
            [unmarshal_BackendStage(v) for v in field] if field is not None else None
        )

    field = data.get("pipeline", None)
    if field is not None:
        args["pipeline"] = unmarshal_Pipeline(field)
    else:
        args["pipeline"] = None

    return PipelineStages(**args)


def unmarshal_PurgeRequest(data: Any) -> PurgeRequest:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'PurgeRequest' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("pipeline_id", None)
    if field is not None:
        args["pipeline_id"] = field

    field = data.get("status", None)
    if field is not None:
        args["status"] = field

    field = data.get("assets", None)
    if field is not None:
        args["assets"] = field
    else:
        args["assets"] = None

    field = data.get("all", None)
    if field is not None:
        args["all"] = field
    else:
        args["all"] = None

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    return PurgeRequest(**args)


def unmarshal_CheckDomainResponse(data: Any) -> CheckDomainResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'CheckDomainResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("is_valid", None)
    if field is not None:
        args["is_valid"] = field

    return CheckDomainResponse(**args)


def unmarshal_CheckLbOriginResponse(data: Any) -> CheckLbOriginResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'CheckLbOriginResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("is_valid", None)
    if field is not None:
        args["is_valid"] = field

    field = data.get("error_type", None)
    if field is not None:
        args["error_type"] = field

    return CheckLbOriginResponse(**args)


def unmarshal_CheckPEMChainResponse(data: Any) -> CheckPEMChainResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'CheckPEMChainResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("is_valid", None)
    if field is not None:
        args["is_valid"] = field

    return CheckPEMChainResponse(**args)


def unmarshal_PlanDetails(data: Any) -> PlanDetails:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'PlanDetails' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("plan_name", None)
    if field is not None:
        args["plan_name"] = field

    field = data.get("package_gb", None)
    if field is not None:
        args["package_gb"] = field

    field = data.get("pipeline_limit", None)
    if field is not None:
        args["pipeline_limit"] = field

    field = data.get("waf_requests", None)
    if field is not None:
        args["waf_requests"] = field

    return PlanDetails(**args)


def unmarshal_PlanUsageDetails(data: Any) -> PlanUsageDetails:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'PlanUsageDetails' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("plan_cost", None)
    if field is not None:
        args["plan_cost"] = unmarshal_Money(field)
    else:
        args["plan_cost"] = None

    return PlanUsageDetails(**args)


def unmarshal_GetBillingResponse(data: Any) -> GetBillingResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'GetBillingResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("current_plan", None)
    if field is not None:
        args["current_plan"] = unmarshal_PlanDetails(field)
    else:
        args["current_plan"] = None

    field = data.get("pipeline_number", None)
    if field is not None:
        args["pipeline_number"] = field

    field = data.get("current_plan_cache_usage", None)
    if field is not None:
        args["current_plan_cache_usage"] = field

    field = data.get("extra_cache_usage", None)
    if field is not None:
        args["extra_cache_usage"] = field

    field = data.get("current_plan_waf_usage", None)
    if field is not None:
        args["current_plan_waf_usage"] = field

    field = data.get("extra_waf_usage", None)
    if field is not None:
        args["extra_waf_usage"] = field

    field = data.get("plan_cost", None)
    if field is not None:
        args["plan_cost"] = unmarshal_Money(field)
    else:
        args["plan_cost"] = None

    field = data.get("extra_pipelines_cost", None)
    if field is not None:
        args["extra_pipelines_cost"] = unmarshal_Money(field)
    else:
        args["extra_pipelines_cost"] = None

    field = data.get("plans_usage_details", None)
    if field is not None:
        args["plans_usage_details"] = (
            {key: unmarshal_PlanUsageDetails(value) for key, value in field.items()}
            if field is not None
            else None
        )

    field = data.get("extra_cache_cost", None)
    if field is not None:
        args["extra_cache_cost"] = unmarshal_Money(field)
    else:
        args["extra_cache_cost"] = None

    field = data.get("extra_waf_cost", None)
    if field is not None:
        args["extra_waf_cost"] = unmarshal_Money(field)
    else:
        args["extra_waf_cost"] = None

    field = data.get("waf_add_on", None)
    if field is not None:
        args["waf_add_on"] = unmarshal_Money(field)
    else:
        args["waf_add_on"] = None

    field = data.get("total_cost", None)
    if field is not None:
        args["total_cost"] = unmarshal_Money(field)
    else:
        args["total_cost"] = None

    return GetBillingResponse(**args)


def unmarshal_ListBackendStagesResponse(data: Any) -> ListBackendStagesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListBackendStagesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("stages", None)
    if field is not None:
        args["stages"] = (
            [unmarshal_BackendStage(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListBackendStagesResponse(**args)


def unmarshal_ListCacheStagesResponse(data: Any) -> ListCacheStagesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListCacheStagesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("stages", None)
    if field is not None:
        args["stages"] = (
            [unmarshal_CacheStage(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListCacheStagesResponse(**args)


def unmarshal_ListDNSStagesResponse(data: Any) -> ListDNSStagesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListDNSStagesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("stages", None)
    if field is not None:
        args["stages"] = (
            [unmarshal_DNSStage(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListDNSStagesResponse(**args)


def unmarshal_ListPipelinesResponse(data: Any) -> ListPipelinesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPipelinesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("pipelines", None)
    if field is not None:
        args["pipelines"] = (
            [unmarshal_Pipeline(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListPipelinesResponse(**args)


def unmarshal_ListPipelinesWithStagesResponse(
    data: Any,
) -> ListPipelinesWithStagesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPipelinesWithStagesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("pipelines", None)
    if field is not None:
        args["pipelines"] = (
            [unmarshal_PipelineStages(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListPipelinesWithStagesResponse(**args)


def unmarshal_ListPlansResponse(data: Any) -> ListPlansResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPlansResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("plans", None)
    if field is not None:
        args["plans"] = (
            [unmarshal_PlanDetails(v) for v in field] if field is not None else None
        )

    return ListPlansResponse(**args)


def unmarshal_ListPurgeRequestsResponse(data: Any) -> ListPurgeRequestsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListPurgeRequestsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("purge_requests", None)
    if field is not None:
        args["purge_requests"] = (
            [unmarshal_PurgeRequest(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListPurgeRequestsResponse(**args)


def unmarshal_ListTLSStagesResponse(data: Any) -> ListTLSStagesResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListTLSStagesResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("stages", None)
    if field is not None:
        args["stages"] = (
            [unmarshal_TLSStage(v) for v in field] if field is not None else None
        )

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    return ListTLSStagesResponse(**args)


def unmarshal_Plan(data: Any) -> Plan:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Plan' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("plan_name", None)
    if field is not None:
        args["plan_name"] = field

    return Plan(**args)


def marshal_CheckDomainRequest(
    request: CheckDomainRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.fqdn is not None:
        output["fqdn"] = request.fqdn

    if request.cname is not None:
        output["cname"] = request.cname

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_ScalewayLb(
    request: ScalewayLb,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.id is not None:
        output["id"] = request.id

    if request.zone is not None:
        output["zone"] = request.zone or defaults.default_zone

    if request.frontend_id is not None:
        output["frontend_id"] = request.frontend_id

    if request.is_ssl is not None:
        output["is_ssl"] = request.is_ssl

    if request.domain_name is not None:
        output["domain_name"] = request.domain_name

    return output


def marshal_CheckLbOriginRequest(
    request: CheckLbOriginRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.lb is not None:
        output["lb"] = marshal_ScalewayLb(request.lb, defaults)

    return output


def marshal_CheckPEMChainRequestSecretChain(
    request: CheckPEMChainRequestSecretChain,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.secret_id is not None:
        output["secret_id"] = request.secret_id

    if request.secret_region is not None:
        output["secret_region"] = request.secret_region

    return output


def marshal_CheckPEMChainRequest(
    request: CheckPEMChainRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("secret", request.secret),
                OneOfPossibility("raw", request.raw),
            ]
        ),
    )

    if request.fqdn is not None:
        output["fqdn"] = request.fqdn

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_ScalewayLbBackendConfig(
    request: ScalewayLbBackendConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.lbs is not None:
        output["lbs"] = [marshal_ScalewayLb(item, defaults) for item in request.lbs]

    return output


def marshal_ScalewayS3BackendConfig(
    request: ScalewayS3BackendConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.bucket_name is not None:
        output["bucket_name"] = request.bucket_name

    if request.bucket_region is not None:
        output["bucket_region"] = request.bucket_region

    if request.is_website is not None:
        output["is_website"] = request.is_website

    return output


def marshal_CreateBackendStageRequest(
    request: CreateBackendStageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("scaleway_s3", request.scaleway_s3),
                OneOfPossibility("scaleway_lb", request.scaleway_lb),
            ]
        ),
    )

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_CreateCacheStageRequest(
    request: CreateCacheStageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("backend_stage_id", request.backend_stage_id),
            ]
        ),
    )

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.fallback_ttl is not None:
        output["fallback_ttl"] = request.fallback_ttl

    return output


def marshal_CreateDNSStageRequest(
    request: CreateDNSStageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("tls_stage_id", request.tls_stage_id),
                OneOfPossibility("cache_stage_id", request.cache_stage_id),
                OneOfPossibility("backend_stage_id", request.backend_stage_id),
            ]
        ),
    )

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.fqdns is not None:
        output["fqdns"] = request.fqdns

    return output


def marshal_CreatePipelineRequest(
    request: CreatePipelineRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("dns_stage_id", request.dns_stage_id),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    if request.description is not None:
        output["description"] = request.description

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    return output


def marshal_CreatePurgeRequestRequest(
    request: CreatePurgeRequestRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("assets", request.assets),
                OneOfPossibility("all", request.all),
            ]
        ),
    )

    if request.pipeline_id is not None:
        output["pipeline_id"] = request.pipeline_id

    return output


def marshal_TLSSecret(
    request: TLSSecret,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.secret_id is not None:
        output["secret_id"] = request.secret_id

    if request.region is not None:
        output["region"] = request.region or defaults.default_region

    return output


def marshal_CreateTLSStageRequest(
    request: CreateTLSStageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("cache_stage_id", request.cache_stage_id),
                OneOfPossibility("backend_stage_id", request.backend_stage_id),
            ]
        ),
    )

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.secrets is not None:
        output["secrets"] = [
            marshal_TLSSecret(item, defaults) for item in request.secrets
        ]

    if request.managed_certificate is not None:
        output["managed_certificate"] = request.managed_certificate

    return output


def marshal_SelectPlanRequest(
    request: SelectPlanRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.plan_name is not None:
        output["plan_name"] = str(request.plan_name)

    return output


def marshal_UpdateBackendStageRequest(
    request: UpdateBackendStageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("scaleway_s3", request.scaleway_s3),
                OneOfPossibility("scaleway_lb", request.scaleway_lb),
            ]
        ),
    )

    return output


def marshal_UpdateCacheStageRequest(
    request: UpdateCacheStageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("backend_stage_id", request.backend_stage_id),
            ]
        ),
    )

    if request.fallback_ttl is not None:
        output["fallback_ttl"] = request.fallback_ttl

    return output


def marshal_UpdateDNSStageRequest(
    request: UpdateDNSStageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("tls_stage_id", request.tls_stage_id),
                OneOfPossibility("cache_stage_id", request.cache_stage_id),
                OneOfPossibility("backend_stage_id", request.backend_stage_id),
            ]
        ),
    )

    if request.fqdns is not None:
        output["fqdns"] = request.fqdns

    return output


def marshal_UpdatePipelineRequest(
    request: UpdatePipelineRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("dns_stage_id", request.dns_stage_id),
            ]
        ),
    )

    if request.name is not None:
        output["name"] = request.name

    if request.description is not None:
        output["description"] = request.description

    return output


def marshal_TLSSecretsConfig(
    request: TLSSecretsConfig,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.tls_secrets is not None:
        output["tls_secrets"] = [
            marshal_TLSSecret(item, defaults) for item in request.tls_secrets
        ]

    return output


def marshal_UpdateTLSStageRequest(
    request: UpdateTLSStageRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("cache_stage_id", request.cache_stage_id),
                OneOfPossibility("backend_stage_id", request.backend_stage_id),
            ]
        ),
    )

    if request.tls_secrets_config is not None:
        output["tls_secrets_config"] = marshal_TLSSecretsConfig(
            request.tls_secrets_config, defaults
        )

    if request.managed_certificate is not None:
        output["managed_certificate"] = request.managed_certificate

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import List, Optional

from scaleway_core.bridge import (
    Money,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class ApplicationType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TYPE = "unknown_type"
    VQE = "vqe"

    def __str__(self) -> str:
        return str(self.value)


class JobStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    WAITING = "waiting"
    ERROR = "error"
    RUNNING = "running"
    COMPLETED = "completed"
    CANCELLING = "cancelling"
    CANCELLED = "cancelled"

    def __str__(self) -> str:
        return str(self.value)


class ListApplicationsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    TYPE_ASC = "type_asc"
    TYPE_DESC = "type_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListJobResultsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_DESC = "created_at_desc"
    CREATED_AT_ASC = "created_at_asc"

    def __str__(self) -> str:
        return str(self.value)


class ListJobsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_DESC = "created_at_desc"
    CREATED_AT_ASC = "created_at_asc"
    STATUS_ASC = "status_asc"
    STATUS_DESC = "status_desc"
    PLATFORM_NAME_ASC = "platform_name_asc"
    PLATFORM_NAME_DESC = "platform_name_desc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    SESSION_NAME_ASC = "session_name_asc"
    SESSION_NAME_DESC = "session_name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListPlatformsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    PROVIDER_NAME_ASC = "provider_name_asc"
    PROVIDER_NAME_DESC = "provider_name_desc"
    TYPE_ASC = "type_asc"
    TYPE_DESC = "type_desc"
    TECHNOLOGY_ASC = "technology_asc"
    TECHNOLOGY_DESC = "technology_desc"
    BACKEND_NAME_ASC = "backend_name_asc"
    BACKEND_NAME_DESC = "backend_name_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListProcessResultsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_DESC = "created_at_desc"
    CREATED_AT_ASC = "created_at_asc"

    def __str__(self) -> str:
        return str(self.value)


class ListProcessesRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_DESC = "created_at_desc"
    CREATED_AT_ASC = "created_at_asc"
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    STARTED_AT_ASC = "started_at_asc"
    STARTED_AT_DESC = "started_at_desc"
    STATUS_ASC = "status_asc"
    STATUS_DESC = "status_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListSessionACLsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    ACCESS_ASC = "access_asc"
    ACCESS_DESC = "access_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListSessionsRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    NAME_ASC = "name_asc"
    NAME_DESC = "name_desc"
    STARTED_AT_ASC = "started_at_asc"
    STARTED_AT_DESC = "started_at_desc"
    STATUS_ASC = "status_asc"
    STATUS_DESC = "status_desc"
    CREATED_AT_DESC = "created_at_desc"
    CREATED_AT_ASC = "created_at_asc"

    def __str__(self) -> str:
        return str(self.value)


class PlatformAvailability(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_AVAILABILITY = "unknown_availability"
    AVAILABLE = "available"
    SHORTAGE = "shortage"
    SCARCE = "scarce"

    def __str__(self) -> str:
        return str(self.value)


class PlatformTechnology(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TECHNOLOGY = "unknown_technology"
    PHOTONIC = "photonic"
    GENERAL_PURPOSE = "general_purpose"

    def __str__(self) -> str:
        return str(self.value)


class PlatformType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_TYPE = "unknown_type"
    SIMULATOR = "simulator"
    QPU = "qpu"

    def __str__(self) -> str:
        return str(self.value)


class ProcessStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    ERROR = "error"
    STARTING = "starting"
    RUNNING = "running"
    COMPLETED = "completed"
    CANCELLING = "cancelling"
    CANCELLED = "cancelled"

    def __str__(self) -> str:
        return str(self.value)


class SessionAccess(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_ACCESS = "unknown_access"
    FULL = "full"
    READ_SESSION = "read_session"
    READ_WRITE_SESSION = "read_write_session"
    READ_JOB_RESULT = "read_job_result"
    READ_JOB_CIRCUIT = "read_job_circuit"
    READ_JOB = "read_job"
    READ_WRITE_JOB = "read_write_job"

    def __str__(self) -> str:
        return str(self.value)


class SessionOriginType(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_ORIGIN_TYPE = "unknown_origin_type"
    CUSTOMER = "customer"
    PROCESS = "process"

    def __str__(self) -> str:
        return str(self.value)


class SessionStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    RUNNING = "running"
    STOPPED = "stopped"
    STARTING = "starting"
    STOPPING = "stopping"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class PlatformHardware:
    name: str
    """
    Product name of the hardware.
    """

    vcpus: int
    """
    Number of vCPUs available.
    """

    gpus: int
    """
    Number of GPUs available (0 if no GPU).
    """

    gpus_network: str
    """
    Network topology of GPUs (PCIe, NVLink...).
    """

    ram: int
    """
    Amount of RAM available in byte.
    """

    vram: int
    """
    Amount of VRAM available in byte (0 if no GPU).
    """


@dataclass
class JobCircuit:
    perceval_circuit: Optional[str]

    qiskit_circuit: Optional[str]


@dataclass
class Application:
    id: str
    """
    Unique ID of the application.
    """

    name: str
    """
    Name of the application.
    """

    type_: ApplicationType
    """
    Type of the application.
    """

    compatible_platform_ids: List[str]
    """
    List of compatible platform (by IDs) able to run this application.
    """

    input_template: str
    """
    JSON format describing the expected input.
    """


@dataclass
class JobResult:
    job_id: str
    """
    ID of the parent job.
    """

    result: Optional[str]
    """
    Result in JSON format.
    """

    url: Optional[str]
    """
    URL to download a large result (optional).
    """

    created_at: Optional[datetime]
    """
    Creation time of the result.
    """


@dataclass
class Job:
    id: str
    """
    Unique ID of the job.
    """

    name: str
    """
    Job name.
    """

    session_id: str
    """
    Session ID in which the job is executed.
    """

    status: JobStatus
    """
    Job status.
    """

    tags: Optional[List[str]]
    """
    Tags of the job.
    """

    created_at: Optional[datetime]
    """
    Time at which the job was created.
    """

    started_at: Optional[datetime]
    """
    Time at which the job was started.
    """

    updated_at: Optional[datetime]
    """
    Time at which the job was updated.
    """

    progress_message: Optional[str]
    """
    Last progress message, if the job has started.
    """

    job_duration: Optional[str]
    """
    Duration of the job, if the job is finished.
    """

    result_distribution: Optional[str]
    """
    Result of the job, if the job is finished.
    """


@dataclass
class Platform:
    id: str
    """
    Unique ID of the platform.
    """

    version: str
    """
    Verison of the platform.
    """

    name: str
    """
    Name of the platform.
    """

    provider_name: str
    """
    Provider name of the platform.
    """

    backend_name: str
    """
    Name of the running backend over the platform (ascella, qsim, aer...).
    """

    type_: PlatformType
    """
    Type of the platform.
    """

    technology: PlatformTechnology
    """
    Technology used by the platform.
    """

    max_qubit_count: int
    """
    Estimated maximum number of qubits supported by the platform.
    """

    availability: PlatformAvailability
    """
    Availability of the platform.
    """

    metadata: str
    """
    Metadata provided by the platform.
    """

    price_per_hour: Optional[Money]
    """
    Price to be payed per hour (excluding free tiers).
    """

    hardware: Optional[PlatformHardware]
    """
    Specifications of the underlying hardware.
    """


@dataclass
class ProcessResult:
    process_id: str
    """
    ID of the parent process.
    """

    result: str
    """
    Result in JSON format.
    """

    created_at: Optional[datetime]
    """
    Creation time of the result.
    """


@dataclass
class Process:
    id: str
    """
    Unique ID of the process.
    """

    name: str
    """
    Name of the process.
    """

    attached_session_ids: List[str]
    """
    List of sessions generated by the process.
    """

    application_id: Optional[str]
    """
    Application ID for which the process has been created.
    """

    status: ProcessStatus
    """
    Status of the process.
    """

    project_id: str
    """
    Project ID in which the process has been created.
    """

    tags: List[str]
    """
    Tags of the process.
    """

    platform_id: Optional[str]
    """
    Platform ID for which the process has been created.
    """

    created_at: Optional[datetime]
    """
    Tme at which the process was created.
    """

    started_at: Optional[datetime]
    """
    Time at which the process started.
    """

    updated_at: Optional[datetime]
    """
    Time at which the process was updated.
    """

    finished_at: Optional[datetime]
    """
    Time at which the process was finished.
    """

    progress: Optional[int]
    """
    Progress of the process, from 0 to 1.
    """

    progress_message: Optional[str]
    """
    Any progress of the process.
    """

    input: Optional[str]
    """
    Input payload of the process as JSON string.
    """


@dataclass
class Session:
    id: str
    """
    Unique ID of the session.
    """

    name: str
    """
    Name of the session.
    """

    platform_id: str
    """
    Platform ID for which the session has been created.
    """

    waiting_job_count: int
    """
    Number of waiting jobs linked to the session.
    """

    created_at: Optional[datetime]
    """
    The time at which the session was created.
    """

    finished_job_count: int
    """
    Number of finished jobs linked to the session.
    """

    status: SessionStatus
    """
    Status of the session.
    """

    project_id: str
    """
    Project ID in which the session has been created.
    """

    deduplication_id: str
    """
    Deduplication ID of the session.
    """

    origin_type: SessionOriginType
    """
    Resource type that creates the session.
    """

    started_at: Optional[datetime]
    """
    The time at which the session started.
    """

    updated_at: Optional[datetime]
    """
    The time at which the session was updated.
    """

    terminated_at: Optional[datetime]
    """
    The time at which the session was terminated.
    """

    max_idle_duration: Optional[str]
    """
    Maximum idle time before the session ends.
    """

    max_duration: Optional[str]
    """
    Maximum duration before the session ends.
    """

    tags: Optional[List[str]]
    """
    Tags of the session.
    """

    origin_id: Optional[str]
    """
    Unique ID of the session's origin resource (if exists).
    """

    progress_message: Optional[str]
    """
    Any progress of the session.
    """


@dataclass
class CancelJobRequest:
    job_id: str
    """
    Unique ID of the job.
    """


@dataclass
class CancelProcessRequest:
    process_id: str
    """
    Unique ID of the process.
    """


@dataclass
class CreateJobRequest:
    name: str
    """
    Name of the job.
    """

    session_id: str
    """
    Session in which the job is executed.
    """

    circuit: JobCircuit
    """
    Quantum circuit that should be executed.
    """

    tags: Optional[List[str]]
    """
    Tags of the job.
    """

    max_duration: Optional[str]
    """
    Maximum duration of the job.
    """


@dataclass
class CreateProcessRequest:
    name: str
    """
    Name of the process.
    """

    project_id: Optional[str]
    """
    ID of the project in which the process was created.
    """

    platform_id: Optional[str]
    """
    ID of the platform for which the process was created.
    """

    application_id: Optional[str]
    """
    ID of the application for which the process was created.
    """

    input: Optional[str]
    """
    Process parameters in JSON format.
    """

    tags: Optional[List[str]]
    """
    Tags of the process.
    """


@dataclass
class CreateSessionRequest:
    platform_id: str
    """
    ID of the Platform for which the session was created.
    """

    project_id: Optional[str]
    """
    ID of the Project in which the session was created.
    """

    name: Optional[str]
    """
    Name of the session.
    """

    max_idle_duration: Optional[str]
    """
    Maximum idle duration before the session ends.
    """

    max_duration: Optional[str]
    """
    Maximum duration before the session ends.
    """

    tags: Optional[List[str]]
    """
    Tags of the session.
    """

    deduplication_id: Optional[str]
    """
    Deduplication ID of the session.
    """


@dataclass
class DeleteJobRequest:
    job_id: str
    """
    Unique ID of the job.
    """


@dataclass
class DeleteProcessRequest:
    process_id: str
    """
    Unique ID of the process.
    """


@dataclass
class DeleteSessionRequest:
    session_id: str
    """
    Unique ID of the session.
    """


@dataclass
class GetApplicationRequest:
    application_id: str
    """
    Unique ID of the application.
    """


@dataclass
class GetJobCircuitRequest:
    job_id: str
    """
    Unique ID of the job.
    """


@dataclass
class GetJobRequest:
    job_id: str
    """
    Unique ID of the job you want to get.
    """


@dataclass
class GetPlatformRequest:
    platform_id: str
    """
    Unique ID of the platform.
    """


@dataclass
class GetProcessRequest:
    process_id: str
    """
    Unique ID of the process.
    """


@dataclass
class GetSessionRequest:
    session_id: str
    """
    Unique ID of the session.
    """


@dataclass
class ListApplicationsRequest:
    name: Optional[str]
    """
    List applications with this name.
    """

    application_type: Optional[ApplicationType]
    """
    List applications with this type.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Maximum number of applications a to return per page.
    """

    order_by: Optional[ListApplicationsRequestOrderBy]
    """
    Sort order of the returned applications.
    """


@dataclass
class ListApplicationsResponse:
    total_count: int
    """
    Total number of applications.
    """

    applications: List[Application]
    """
    List of applications.
    """


@dataclass
class ListJobResultsRequest:
    job_id: str
    """
    ID of the job.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Maximum number of results to return per page.
    """

    order_by: Optional[ListJobResultsRequestOrderBy]
    """
    Sort order of the returned results.
    """


@dataclass
class ListJobResultsResponse:
    total_count: int
    """
    Total number of results.
    """

    job_results: List[JobResult]
    """
    List of results.
    """


@dataclass
class ListJobsRequest:
    tags: Optional[List[str]]
    """
    List jobs with these tags.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Maximum number of jobs to return per page.
    """

    order_by: Optional[ListJobsRequestOrderBy]
    """
    Sort order of the returned jobs.
    """

    session_id: Optional[str]

    project_id: Optional[str]


@dataclass
class ListJobsResponse:
    total_count: int
    """
    Total number of jobs.
    """

    jobs: List[Job]
    """
    List of jobs.
    """


@dataclass
class ListPlatformsRequest:
    provider_name: Optional[str]
    """
    List platforms with this provider name.
    """

    backend_name: Optional[str]
    """
    List platforms with this backend name.
    """

    name: Optional[str]
    """
    List platforms with this name.
    """

    platform_type: Optional[PlatformType]
    """
    List platforms with this type.
    """

    platform_technology: Optional[PlatformTechnology]
    """
    List platforms with this technology.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Maximum number of platforms to return per page.
    """

    order_by: Optional[ListPlatformsRequestOrderBy]
    """
    Sort order of the returned platforms.
    """


@dataclass
class ListPlatformsResponse:
    total_count: int
    """
    Total number of platforms.
    """

    platforms: List[Platform]
    """
    List of platforms.
    """


@dataclass
class ListProcessResultsRequest:
    process_id: str
    """
    ID of the process.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Maximum number of results to return per page.
    """

    order_by: Optional[ListProcessResultsRequestOrderBy]
    """
    Sort order of the returned results.
    """


@dataclass
class ListProcessResultsResponse:
    total_count: int
    """
    Total number of results.
    """

    process_results: List[ProcessResult]
    """
    List of results.
    """


@dataclass
class ListProcessesRequest:
    application_id: Optional[str]
    """
    List processes that have been created for this application.
    """

    tags: Optional[List[str]]
    """
    List processes with these tags.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Maximum number of processes to return per page.
    """

    order_by: Optional[ListProcessesRequestOrderBy]
    """
    Sort order of the returned processes.
    """

    project_id: Optional[str]
    """
    List processes belonging to this project ID.
    """


@dataclass
class ListProcessesResponse:
    total_count: int
    """
    Total number of processes.
    """

    processes: List[Process]
    """
    List of processes.
    """


@dataclass
class ListSessionACLsRequest:
    session_id: str

    page: Optional[int]

    page_size: Optional[int]

    order_by: Optional[ListSessionACLsRequestOrderBy]


@dataclass
class ListSessionACLsResponse:
    total_count: int

    acls: List[SessionAccess]


@dataclass
class ListSessionsRequest:
    platform_id: Optional[str]
    """
    List sessions that have been created for this platform.
    """

    tags: Optional[List[str]]
    """
    List sessions with these tags.
    """

    page: Optional[int]
    """
    Page number.
    """

    page_size: Optional[int]
    """
    Maximum number of sessions to return per page.
    """

    order_by: Optional[ListSessionsRequestOrderBy]
    """
    Sort order of the returned sessions.
    """

    project_id: Optional[str]
    """
    List sessions belonging to this project ID.
    """


@dataclass
class ListSessionsResponse:
    total_count: int
    """
    Total number of sessions.
    """

    sessions: List[Session]
    """
    List of sessions.
    """


@dataclass
class TerminateSessionRequest:
    session_id: str
    """
    Unique ID of the session.
    """


@dataclass
class UpdateJobRequest:
    job_id: str
    """
    Unique ID of the job.
    """

    name: Optional[str]
    """
    Name of the job.
    """

    tags: Optional[List[str]]
    """
    Tags of the job.
    """


@dataclass
class UpdateProcessRequest:
    process_id: str
    """
    Unique ID of the process.
    """

    name: Optional[str]
    """
    Name of the process.
    """

    tags: Optional[List[str]]
    """
    Tags of the process.
    """


@dataclass
class UpdateSessionRequest:
    session_id: str
    """
    Unique ID of the session.
    """

    name: Optional[str]
    """
    Name of the session.
    """

    max_idle_duration: Optional[str]
    """
    Maximum idle duration before the session ends.
    """

    max_duration: Optional[str]
    """
    Maximum time before the session ends.
    """

    tags: Optional[List[str]]
    """
    Tags of the session.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import ContractType
from .types import ListContractSignaturesRequestOrderBy
from .types import ListProjectsRequestOrderBy
from .types import QualificationArchitectureType
from .types import QualificationHostApplicationSubUseCase
from .types import QualificationHostWebsiteSubUseCase
from .types import Contract
from .types import QualificationAiMachine
from .types import QualificationArchiveData
from .types import QualificationContainer
from .types import QualificationDeploySoftware
from .types import QualificationHostApplication
from .types import QualificationHostWebsite
from .types import QualificationOtherUseCase
from .types import QualificationSetScalewayEnvironment
from .types import QualificationShareData
from .types import ContractSignature
from .types import Project
from .types import Qualification
from .types import CheckContractSignatureResponse
from .types import ContractApiCheckContractSignatureRequest
from .types import ContractApiCreateContractSignatureRequest
from .types import ContractApiDownloadContractSignatureRequest
from .types import ContractApiListContractSignaturesRequest
from .types import ContractApiValidateContractSignatureRequest
from .types import ListContractSignaturesResponse
from .types import ListProjectsResponse
from .types import ProjectApiCreateProjectRequest
from .types import ProjectApiDeleteProjectRequest
from .types import ProjectApiGetProjectRequest
from .types import ProjectApiListProjectsRequest
from .types import ProjectApiSetProjectQualificationRequest
from .types import ProjectApiUpdateProjectRequest
from .types import ProjectQualification
from .api import AccountV3ContractAPI
from .api import AccountV3ProjectAPI

__all__ = [
    "ContractType",
    "ListContractSignaturesRequestOrderBy",
    "ListProjectsRequestOrderBy",
    "QualificationArchitectureType",
    "QualificationHostApplicationSubUseCase",
    "QualificationHostWebsiteSubUseCase",
    "Contract",
    "QualificationAiMachine",
    "QualificationArchiveData",
    "QualificationContainer",
    "QualificationDeploySoftware",
    "QualificationHostApplication",
    "QualificationHostWebsite",
    "QualificationOtherUseCase",
    "QualificationSetScalewayEnvironment",
    "QualificationShareData",
    "ContractSignature",
    "Project",
    "Qualification",
    "CheckContractSignatureResponse",
    "ContractApiCheckContractSignatureRequest",
    "ContractApiCreateContractSignatureRequest",
    "ContractApiDownloadContractSignatureRequest",
    "ContractApiListContractSignaturesRequest",
    "ContractApiValidateContractSignatureRequest",
    "ListContractSignaturesResponse",
    "ListProjectsResponse",
    "ProjectApiCreateProjectRequest",
    "ProjectApiDeleteProjectRequest",
    "ProjectApiGetProjectRequest",
    "ProjectApiListProjectsRequest",
    "ProjectApiSetProjectQualificationRequest",
    "ProjectApiUpdateProjectRequest",
    "ProjectQualification",
    "AccountV3ContractAPI",
    "AccountV3ProjectAPI",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import IPReverseStatus
from .types import IPVersion
from .types import ListServerEventsRequestOrderBy
from .types import ListServerPrivateNetworksRequestOrderBy
from .types import ListServersRequestOrderBy
from .types import ListSettingsRequestOrderBy
from .types import OfferStock
from .types import OfferSubscriptionPeriod
from .types import SchemaFilesystemFormat
from .types import SchemaPartitionLabel
from .types import SchemaPoolType
from .types import SchemaRAIDLevel
from .types import ServerBootType
from .types import ServerInstallStatus
from .content import SERVER_INSTALL_TRANSIENT_STATUSES
from .types import ServerOptionOptionStatus
from .types import ServerPingStatus
from .types import ServerPrivateNetworkStatus
from .content import SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES
from .types import ServerStatus
from .content import SERVER_TRANSIENT_STATUSES
from .types import SettingType
from .types import SchemaPartition
from .types import SchemaPool
from .types import SchemaDisk
from .types import SchemaFilesystem
from .types import SchemaRAID
from .types import SchemaZFS
from .types import CertificationOption
from .types import LicenseOption
from .types import PrivateNetworkOption
from .types import PublicBandwidthOption
from .types import RemoteAccessOption
from .types import Schema
from .types import OSOSField
from .types import CPU
from .types import Disk
from .types import GPU
from .types import Memory
from .types import OfferOptionOffer
from .types import PersistentMemory
from .types import RaidController
from .types import IP
from .types import ServerInstall
from .types import ServerOption
from .types import ServerRescueServer
from .types import CreateServerRequestInstall
from .types import OS
from .types import Offer
from .types import Option
from .types import ServerEvent
from .types import ServerPrivateNetwork
from .types import Server
from .types import Setting
from .types import AddOptionServerRequest
from .types import BMCAccess
from .types import CreateServerRequest
from .types import DeleteOptionServerRequest
from .types import DeleteServerRequest
from .types import GetBMCAccessRequest
from .types import GetDefaultPartitioningSchemaRequest
from .types import GetOSRequest
from .types import GetOfferRequest
from .types import GetOptionRequest
from .types import GetServerMetricsRequest
from .types import GetServerMetricsResponse
from .types import GetServerRequest
from .types import InstallServerRequest
from .types import ListOSRequest
from .types import ListOSResponse
from .types import ListOffersRequest
from .types import ListOffersResponse
from .types import ListOptionsRequest
from .types import ListOptionsResponse
from .types import ListServerEventsRequest
from .types import ListServerEventsResponse
from .types import ListServerPrivateNetworksResponse
from .types import ListServersRequest
from .types import ListServersResponse
from .types import ListSettingsRequest
from .types import ListSettingsResponse
from .types import MigrateServerToMonthlyOfferRequest
from .types import PrivateNetworkApiAddServerPrivateNetworkRequest
from .types import PrivateNetworkApiDeleteServerPrivateNetworkRequest
from .types import PrivateNetworkApiListServerPrivateNetworksRequest
from .types import PrivateNetworkApiSetServerPrivateNetworksRequest
from .types import RebootServerRequest
from .types import SetServerPrivateNetworksResponse
from .types import StartBMCAccessRequest
from .types import StartServerRequest
from .types import StopBMCAccessRequest
from .types import StopServerRequest
from .types import UpdateIPRequest
from .types import UpdateServerRequest
from .types import UpdateSettingRequest
from .types import ValidatePartitioningSchemaRequest
from .api import BaremetalV1API
from .api import BaremetalV1PrivateNetworkAPI

__all__ = [
    "IPReverseStatus",
    "IPVersion",
    "ListServerEventsRequestOrderBy",
    "ListServerPrivateNetworksRequestOrderBy",
    "ListServersRequestOrderBy",
    "ListSettingsRequestOrderBy",
    "OfferStock",
    "OfferSubscriptionPeriod",
    "SchemaFilesystemFormat",
    "SchemaPartitionLabel",
    "SchemaPoolType",
    "SchemaRAIDLevel",
    "ServerBootType",
    "ServerInstallStatus",
    "SERVER_INSTALL_TRANSIENT_STATUSES",
    "ServerOptionOptionStatus",
    "ServerPingStatus",
    "ServerPrivateNetworkStatus",
    "SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES",
    "ServerStatus",
    "SERVER_TRANSIENT_STATUSES",
    "SettingType",
    "SchemaPartition",
    "SchemaPool",
    "SchemaDisk",
    "SchemaFilesystem",
    "SchemaRAID",
    "SchemaZFS",
    "CertificationOption",
    "LicenseOption",
    "PrivateNetworkOption",
    "PublicBandwidthOption",
    "RemoteAccessOption",
    "Schema",
    "OSOSField",
    "CPU",
    "Disk",
    "GPU",
    "Memory",
    "OfferOptionOffer",
    "PersistentMemory",
    "RaidController",
    "IP",
    "ServerInstall",
    "ServerOption",
    "ServerRescueServer",
    "CreateServerRequestInstall",
    "OS",
    "Offer",
    "Option",
    "ServerEvent",
    "ServerPrivateNetwork",
    "Server",
    "Setting",
    "AddOptionServerRequest",
    "BMCAccess",
    "CreateServerRequest",
    "DeleteOptionServerRequest",
    "DeleteServerRequest",
    "GetBMCAccessRequest",
    "GetDefaultPartitioningSchemaRequest",
    "GetOSRequest",
    "GetOfferRequest",
    "GetOptionRequest",
    "GetServerMetricsRequest",
    "GetServerMetricsResponse",
    "GetServerRequest",
    "InstallServerRequest",
    "ListOSRequest",
    "ListOSResponse",
    "ListOffersRequest",
    "ListOffersResponse",
    "ListOptionsRequest",
    "ListOptionsResponse",
    "ListServerEventsRequest",
    "ListServerEventsResponse",
    "ListServerPrivateNetworksResponse",
    "ListServersRequest",
    "ListServersResponse",
    "ListSettingsRequest",
    "ListSettingsResponse",
    "MigrateServerToMonthlyOfferRequest",
    "PrivateNetworkApiAddServerPrivateNetworkRequest",
    "PrivateNetworkApiDeleteServerPrivateNetworkRequest",
    "PrivateNetworkApiListServerPrivateNetworksRequest",
    "PrivateNetworkApiSetServerPrivateNetworksRequest",
    "RebootServerRequest",
    "SetServerPrivateNetworksResponse",
    "StartBMCAccessRequest",
    "StartServerRequest",
    "StopBMCAccessRequest",
    "StopServerRequest",
    "UpdateIPRequest",
    "UpdateServerRequest",
    "UpdateSettingRequest",
    "ValidatePartitioningSchemaRequest",
    "BaremetalV1API",
    "BaremetalV1PrivateNetworkAPI",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from typing import List

from .types import (
    ServerPrivateNetworkServerStatus,
    ServerPrivateNetworkStatus,
    ServerStatus,
)

SERVER_PRIVATE_NETWORK_SERVER_TRANSIENT_STATUSES: List[
    ServerPrivateNetworkServerStatus
] = [
    ServerPrivateNetworkServerStatus.ATTACHING,
    ServerPrivateNetworkServerStatus.DETACHING,
]
"""
Lists transient statutes of the enum :class:`ServerPrivateNetworkServerStatus <ServerPrivateNetworkServerStatus>`.
"""
SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES: List[ServerPrivateNetworkStatus] = [
    ServerPrivateNetworkStatus.VPC_UPDATING,
]
"""
Lists transient statutes of the enum :class:`ServerPrivateNetworkStatus <ServerPrivateNetworkStatus>`.
"""
SERVER_TRANSIENT_STATUSES: List[ServerStatus] = [
    ServerStatus.STARTING,
    ServerStatus.REBOOTING,
    ServerStatus.UPDATING,
    ServerStatus.LOCKING,
    ServerStatus.UNLOCKING,
    ServerStatus.REINSTALLING,
    ServerStatus.BUSY,
]
"""
Lists transient statutes of the enum :class:`ServerStatus <ServerStatus>`.
"""



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import Any, Dict
from dateutil import parser

from scaleway_core.profile import ProfileDefaults
from scaleway_core.utils import (
    OneOfPossibility,
    resolve_one_of,
)
from .types import (
    Resource,
    Reverse,
    Source,
    IP,
    ListIPsResponse,
    CustomResource,
    AttachIPRequest,
    BookIPRequest,
    DetachIPRequest,
    MoveIPRequest,
    ReleaseIPSetRequest,
    UpdateIPRequest,
)


def unmarshal_Resource(data: Any) -> Resource:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Resource' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("type", None)
    if field is not None:
        args["type_"] = field

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("mac_address", None)
    if field is not None:
        args["mac_address"] = field
    else:
        args["mac_address"] = None

    field = data.get("name", None)
    if field is not None:
        args["name"] = field
    else:
        args["name"] = None

    return Resource(**args)


def unmarshal_Reverse(data: Any) -> Reverse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Reverse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("hostname", None)
    if field is not None:
        args["hostname"] = field

    field = data.get("address", None)
    if field is not None:
        args["address"] = field
    else:
        args["address"] = None

    return Reverse(**args)


def unmarshal_Source(data: Any) -> Source:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'Source' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("zonal", None)
    if field is not None:
        args["zonal"] = field
    else:
        args["zonal"] = None

    field = data.get("private_network_id", None)
    if field is not None:
        args["private_network_id"] = field
    else:
        args["private_network_id"] = None

    field = data.get("subnet_id", None)
    if field is not None:
        args["subnet_id"] = field
    else:
        args["subnet_id"] = None

    return Source(**args)


def unmarshal_IP(data: Any) -> IP:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'IP' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("id", None)
    if field is not None:
        args["id"] = field

    field = data.get("address", None)
    if field is not None:
        args["address"] = field

    field = data.get("project_id", None)
    if field is not None:
        args["project_id"] = field

    field = data.get("is_ipv6", None)
    if field is not None:
        args["is_ipv6"] = field

    field = data.get("tags", None)
    if field is not None:
        args["tags"] = field

    field = data.get("reverses", None)
    if field is not None:
        args["reverses"] = (
            [unmarshal_Reverse(v) for v in field] if field is not None else None
        )

    field = data.get("region", None)
    if field is not None:
        args["region"] = field

    field = data.get("created_at", None)
    if field is not None:
        args["created_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["created_at"] = None

    field = data.get("updated_at", None)
    if field is not None:
        args["updated_at"] = parser.isoparse(field) if isinstance(field, str) else field
    else:
        args["updated_at"] = None

    field = data.get("source", None)
    if field is not None:
        args["source"] = unmarshal_Source(field)
    else:
        args["source"] = None

    field = data.get("resource", None)
    if field is not None:
        args["resource"] = unmarshal_Resource(field)
    else:
        args["resource"] = None

    field = data.get("zone", None)
    if field is not None:
        args["zone"] = field
    else:
        args["zone"] = None

    return IP(**args)


def unmarshal_ListIPsResponse(data: Any) -> ListIPsResponse:
    if not isinstance(data, dict):
        raise TypeError(
            "Unmarshalling the type 'ListIPsResponse' failed as data isn't a dictionary."
        )

    args: Dict[str, Any] = {}

    field = data.get("total_count", None)
    if field is not None:
        args["total_count"] = field

    field = data.get("ips", None)
    if field is not None:
        args["ips"] = [unmarshal_IP(v) for v in field] if field is not None else None

    return ListIPsResponse(**args)


def marshal_CustomResource(
    request: CustomResource,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.mac_address is not None:
        output["mac_address"] = request.mac_address

    if request.name is not None:
        output["name"] = request.name

    return output


def marshal_AttachIPRequest(
    request: AttachIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.resource is not None:
        output["resource"] = marshal_CustomResource(request.resource, defaults)

    return output


def marshal_Source(
    request: Source,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}
    output.update(
        resolve_one_of(
            [
                OneOfPossibility("zonal", request.zonal),
                OneOfPossibility("private_network_id", request.private_network_id),
                OneOfPossibility("subnet_id", request.subnet_id),
            ]
        ),
    )

    return output


def marshal_BookIPRequest(
    request: BookIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.source is not None:
        output["source"] = marshal_Source(request.source, defaults)

    if request.is_ipv6 is not None:
        output["is_ipv6"] = request.is_ipv6

    if request.project_id is not None:
        output["project_id"] = request.project_id or defaults.default_project_id

    if request.address is not None:
        output["address"] = request.address

    if request.tags is not None:
        output["tags"] = request.tags

    if request.resource is not None:
        output["resource"] = marshal_CustomResource(request.resource, defaults)

    return output


def marshal_DetachIPRequest(
    request: DetachIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.resource is not None:
        output["resource"] = marshal_CustomResource(request.resource, defaults)

    return output


def marshal_MoveIPRequest(
    request: MoveIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.from_resource is not None:
        output["from_resource"] = marshal_CustomResource(
            request.from_resource, defaults
        )

    if request.to_resource is not None:
        output["to_resource"] = marshal_CustomResource(request.to_resource, defaults)

    return output


def marshal_ReleaseIPSetRequest(
    request: ReleaseIPSetRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.ip_ids is not None:
        output["ip_ids"] = request.ip_ids

    return output


def marshal_Reverse(
    request: Reverse,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.hostname is not None:
        output["hostname"] = request.hostname

    if request.address is not None:
        output["address"] = request.address

    return output


def marshal_UpdateIPRequest(
    request: UpdateIPRequest,
    defaults: ProfileDefaults,
) -> Dict[str, Any]:
    output: Dict[str, Any] = {}

    if request.tags is not None:
        output["tags"] = request.tags

    if request.reverses is not None:
        output["reverses"] = [
            marshal_Reverse(item, defaults) for item in request.reverses
        ]

    return output



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import ListServerPrivateNetworksRequestOrderBy
from .types import ServerPrivateNetworkStatus
from .content import SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES
from .types import ServerPrivateNetwork
from .types import ListServerPrivateNetworksResponse
from .types import PrivateNetworkApiAddServerPrivateNetworkRequest
from .types import PrivateNetworkApiDeleteServerPrivateNetworkRequest
from .types import PrivateNetworkApiListServerPrivateNetworksRequest
from .types import PrivateNetworkApiSetServerPrivateNetworksRequest
from .types import SetServerPrivateNetworksResponse
from .api import BaremetalV3PrivateNetworkAPI

__all__ = [
    "ListServerPrivateNetworksRequestOrderBy",
    "ServerPrivateNetworkStatus",
    "SERVER_PRIVATE_NETWORK_TRANSIENT_STATUSES",
    "ServerPrivateNetwork",
    "ListServerPrivateNetworksResponse",
    "PrivateNetworkApiAddServerPrivateNetworkRequest",
    "PrivateNetworkApiDeleteServerPrivateNetworkRequest",
    "PrivateNetworkApiListServerPrivateNetworksRequest",
    "PrivateNetworkApiSetServerPrivateNetworksRequest",
    "SetServerPrivateNetworksResponse",
    "BaremetalV3PrivateNetworkAPI",
]



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from __future__ import annotations

from dataclasses import dataclass
from datetime import datetime
from enum import Enum
from typing import Dict, List, Optional

from scaleway_core.bridge import (
    Zone as ScwZone,
)
from scaleway_core.utils import (
    StrEnumMeta,
)


class CommitmentType(str, Enum, metaclass=StrEnumMeta):
    DURATION_24H = "duration_24h"
    RENEWED_MONTHLY = "renewed_monthly"
    NONE = "none"

    def __str__(self) -> str:
        return str(self.value)


class ConnectivityDiagnosticActionType(str, Enum, metaclass=StrEnumMeta):
    REBOOT_SERVER = "reboot_server"
    REINSTALL_SERVER = "reinstall_server"

    def __str__(self) -> str:
        return str(self.value)


class ConnectivityDiagnosticDiagnosticStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    PROCESSING = "processing"
    ERROR = "error"
    COMPLETED = "completed"

    def __str__(self) -> str:
        return str(self.value)


class ListServerPrivateNetworksRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"
    UPDATED_AT_ASC = "updated_at_asc"
    UPDATED_AT_DESC = "updated_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ListServersRequestOrderBy(str, Enum, metaclass=StrEnumMeta):
    CREATED_AT_ASC = "created_at_asc"
    CREATED_AT_DESC = "created_at_desc"

    def __str__(self) -> str:
        return str(self.value)


class ServerPrivateNetworkServerStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    ATTACHING = "attaching"
    ATTACHED = "attached"
    ERROR = "error"
    DETACHING = "detaching"
    LOCKED = "locked"

    def __str__(self) -> str:
        return str(self.value)


class ServerPrivateNetworkStatus(str, Enum, metaclass=StrEnumMeta):
    VPC_UNKNOWN_STATUS = "vpc_unknown_status"
    VPC_ENABLED = "vpc_enabled"
    VPC_UPDATING = "vpc_updating"
    VPC_DISABLED = "vpc_disabled"

    def __str__(self) -> str:
        return str(self.value)


class ServerStatus(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STATUS = "unknown_status"
    STARTING = "starting"
    READY = "ready"
    ERROR = "error"
    REBOOTING = "rebooting"
    UPDATING = "updating"
    LOCKING = "locking"
    LOCKED = "locked"
    UNLOCKING = "unlocking"
    REINSTALLING = "reinstalling"
    BUSY = "busy"

    def __str__(self) -> str:
        return str(self.value)


class ServerTypeStock(str, Enum, metaclass=StrEnumMeta):
    UNKNOWN_STOCK = "unknown_stock"
    NO_STOCK = "no_stock"
    LOW_STOCK = "low_stock"
    HIGH_STOCK = "high_stock"

    def __str__(self) -> str:
        return str(self.value)


@dataclass
class OS:
    id: str
    """
    Unique ID of the OS.
    """

    name: str
    """
    OS name.
    """

    label: str
    """
    OS name as it should be displayed.
    """

    image_url: str
    """
    URL of the image.
    """

    family: str
    """
    The OS family to which this OS belongs, eg. 13 or 14.
    """

    is_beta: bool
    """
    Describes if the OS is in beta.
    """

    version: str
    """
    The OS version number, eg. Sonoma has version number 14.3.
    """

    xcode_version: str
    """
    The current xcode version for this OS.
    """

    compatible_server_types: List[str]
    """
    List of compatible server types.
    """


@dataclass
class ServerTypeCPU:
    name: str

    core_count: int

    frequency: int


@dataclass
class ServerTypeDisk:
    capacity: int

    type_: str


@dataclass
class ServerTypeGPU:
    count: int


@dataclass
class ServerTypeMemory:
    capacity: int

    type_: str


@dataclass
class ServerTypeNetwork:
    public_bandwidth_bps: int


@dataclass
class Commitment:
    type_: CommitmentType

    cancelled: bool


@dataclass
class ConnectivityDiagnosticServerHealth:
    is_server_alive: bool

    is_agent_alive: bool

    is_mdm_alive: bool

    is_ssh_port_up: bool

    is_vnc_port_up: bool

    last_checkin_date: Optional[datetime]


@dataclass
class ServerPrivateNetwork:
    id: str
    """
    ID of the Server-to-Private Network mapping.
    """

    project_id: str
    """
    Private Network Project ID.
    """

    server_id: str
    """
    Apple silicon server ID.
    """

    private_network_id: str
    """
    Private Network ID.
    """

    status: ServerPrivateNetworkServerStatus
    """
    Configuration status of the Private Network.
    """

    ipam_ip_ids: List[str]
    """
    IPAM IP IDs of the server, if it has any.
    """

    vlan: Optional[int]
    """
    ID of the VLAN associated with the Private Network.
    """

    created_at: Optional[datetime]
    """
    Private Network creation date.
    """

    updated_at: Optional[datetime]
    """
    Date the Private Network was last modified.
    """


@dataclass
class ServerType:
    name: str
    """
    Name of the type.
    """

    stock: ServerTypeStock
    """
    Current stock.
    """

    cpu: Optional[ServerTypeCPU]
    """
    CPU description.
    """

    disk: Optional[ServerTypeDisk]
    """
    Size of the local disk of the server.
    """

    memory: Optional[ServerTypeMemory]
    """
    Size of memory available.
    """

    minimum_lease_duration: Optional[str]
    """
    Minimum duration of the lease in seconds (example. 3.4s).
    """

    gpu: Optional[ServerTypeGPU]
    """
    GPU description.
    """

    network: Optional[ServerTypeNetwork]
    """
    Network description.
    """

    default_os: Optional[OS]
    """
    The default OS for this server type.
    """


@dataclass
class Server:
    id: str
    """
    UUID of the server.
    """

    type_: str
    """
    Type of the server.
    """

    name: str
    """
    Name of the server.
    """

    project_id: str
    """
    Project this server is associated with.
    """

    organization_id: str
    """
    Organization this server is associated with.
    """

    ip: str
    """
    IPv4 address of the server.
    """

    vnc_url: str
    """
    Vnc:// URL to access Apple Remote Desktop.
    """

    ssh_username: str
    """
    SSH Username for remote shell.
    """

    sudo_password: str
    """
    Admin password required to execute commands.
    """

    vnc_port: int
    """
    VNC port to use for remote desktop connection.
    """

    status: ServerStatus
    """
    Current status of the server.
    """

    os: Optional[OS]
    """
    Initially installed OS, this does not necessarily reflect the current OS version.
    """

    created_at: Optional[datetime]
    """
    Date on which the server was created.
    """

    updated_at: Optional[datetime]
    """
    Date on which the server was last updated.
    """

    deletable_at: Optional[datetime]
    """
    Date from which the server can be deleted.
    """

    deletion_scheduled: bool
    """
    Set to true to mark the server for automatic deletion depending on `deletable_at` date. Set to false to cancel an existing deletion schedule. Leave unset otherwise.
    """

    zone: ScwZone
    """
    Zone of the server.
    """

    delivered: bool
    """
    Set to true once the server has completed its provisioning steps and is ready to use. Some OS configurations might require a reinstallation of the server before delivery depending on the available stock. A reinstallation after the initial delivery will not change this flag and can be tracked using the server status.
    """

    vpc_status: ServerPrivateNetworkStatus
    """
    Activation status of optional Private Network feature support for this server.
    """

    commitment: Optional[Commitment]
    """
    Commitment scheme applied to this server.
    """


@dataclass
class CommitmentTypeValue:
    commitment_type: CommitmentType


@dataclass
class ConnectivityDiagnostic:
    id: str

    status: ConnectivityDiagnosticDiagnosticStatus

    is_healthy: bool

    supported_actions: List[ConnectivityDiagnosticActionType]

    error_message: str

    health_details: Optional[ConnectivityDiagnosticServerHealth]


@dataclass
class CreateServerRequest:
    type_: str
    """
    Create a server of the given type.
    """

    enable_vpc: bool
    """
    Activate the Private Network feature for this server. This feature is configured through the Apple Silicon - Private Networks API.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Create a server with this given name.
    """

    project_id: Optional[str]
    """
    Create a server in the given project ID.
    """

    os_id: Optional[str]
    """
    Create a server & install the given os_id, when no os_id provided the default OS for this server type is chosen. Requesting a non-default OS will induce an extended delivery time.
    """

    commitment_type: Optional[CommitmentType]
    """
    Activate commitment for this server. If not specified, there is a 24h commitment due to Apple licensing (commitment_type `duration_24h`). It can be updated with the Update Server request. Available commitment depends on server type.
    """


@dataclass
class DeleteServerRequest:
    server_id: str
    """
    UUID of the server you want to delete.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetConnectivityDiagnosticRequest:
    diagnostic_id: str

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetOSRequest:
    os_id: str
    """
    UUID of the OS you want to get.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetServerRequest:
    server_id: str
    """
    UUID of the server you want to get.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class GetServerTypeRequest:
    server_type: str
    """
    Server type identifier.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ListOSRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    page: Optional[int]
    """
    Positive integer to choose the page to return.
    """

    page_size: Optional[int]
    """
    Positive integer lower or equal to 100 to select the number of items to return.
    """

    server_type: Optional[str]
    """
    List of compatible server types.
    """

    name: Optional[str]
    """
    Filter OS by name (note that "11.1" will return "11.1.2" and "11.1" but not "12")).
    """


@dataclass
class ListOSResponse:
    total_count: int
    """
    Total number of OS.
    """

    os: List[OS]
    """
    List of OS.
    """


@dataclass
class ListServerPrivateNetworksResponse:
    server_private_networks: List[ServerPrivateNetwork]

    total_count: int


@dataclass
class ListServerTypesRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ListServerTypesResponse:
    server_types: List[ServerType]
    """
    Available server types.
    """


@dataclass
class ListServersRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListServersRequestOrderBy]
    """
    Sort order of the returned servers.
    """

    project_id: Optional[str]
    """
    Only list servers of this project ID.
    """

    organization_id: Optional[str]
    """
    Only list servers of this Organization ID.
    """

    page: Optional[int]
    """
    Positive integer to choose the page to return.
    """

    page_size: Optional[int]
    """
    Positive integer lower or equal to 100 to select the number of items to return.
    """


@dataclass
class ListServersResponse:
    total_count: int
    """
    Total number of servers.
    """

    servers: List[Server]
    """
    Paginated returned servers.
    """


@dataclass
class PrivateNetworkApiAddServerPrivateNetworkRequest:
    server_id: str
    """
    ID of the server.
    """

    private_network_id: str
    """
    ID of the Private Network.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    ipam_ip_ids: Optional[List[str]]
    """
    IPAM IDs of IPs to attach to the server.
    """


@dataclass
class PrivateNetworkApiDeleteServerPrivateNetworkRequest:
    server_id: str
    """
    ID of the server.
    """

    private_network_id: str
    """
    ID of the Private Network.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class PrivateNetworkApiGetServerPrivateNetworkRequest:
    server_id: str

    private_network_id: str

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class PrivateNetworkApiListServerPrivateNetworksRequest:
    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    order_by: Optional[ListServerPrivateNetworksRequestOrderBy]
    """
    Sort order for the returned Private Networks.
    """

    page: Optional[int]
    """
    Page number for the returned Private Networks.
    """

    page_size: Optional[int]
    """
    Maximum number of Private Networks per page.
    """

    server_id: Optional[str]
    """
    Filter Private Networks by server ID.
    """

    private_network_id: Optional[str]
    """
    Filter Private Networks by Private Network ID.
    """

    organization_id: Optional[str]
    """
    Filter Private Networks by Organization ID.
    """

    project_id: Optional[str]
    """
    Filter Private Networks by Project ID.
    """

    ipam_ip_ids: Optional[List[str]]
    """
    Filter Private Networks by IPAM IP IDs.
    """


@dataclass
class PrivateNetworkApiSetServerPrivateNetworksRequest:
    server_id: str
    """
    ID of the server.
    """

    per_private_network_ipam_ip_ids: Dict[str, List[str]]
    """
    Object where the keys are the IDs of Private Networks and the values are arrays of IPAM IDs representing the IPs to assign to this Apple silicon server on the Private Network. If the array supplied for a Private Network is empty, the next available IP from the Private Network's CIDR block will automatically be used for attachment.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class RebootServerRequest:
    server_id: str
    """
    UUID of the server you want to reboot.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class ReinstallServerRequest:
    server_id: str
    """
    UUID of the server you want to reinstall.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    os_id: Optional[str]
    """
    Reinstall the server with the target OS, when no os_id provided the default OS for the server type is used.
    """


@dataclass
class SetServerPrivateNetworksResponse:
    server_private_networks: List[ServerPrivateNetwork]


@dataclass
class StartConnectivityDiagnosticRequest:
    server_id: str

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """


@dataclass
class StartConnectivityDiagnosticResponse:
    diagnostic_id: str


@dataclass
class UpdateServerRequest:
    server_id: str
    """
    UUID of the server you want to update.
    """

    zone: Optional[ScwZone]
    """
    Zone to target. If none is passed will use default zone from the config.
    """

    name: Optional[str]
    """
    Updated name for your server.
    """

    schedule_deletion: Optional[bool]
    """
    Specify whether the server should be flagged for automatic deletion.
    """

    enable_vpc: Optional[bool]
    """
    Activate or deactivate Private Network support for this server.
    """

    commitment_type: Optional[CommitmentTypeValue]
    """
    Change commitment. Use 'none' to automatically cancel a renewing commitment.
    """



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.

from typing import List, Optional

from scaleway_core.api import API
from scaleway_core.bridge import (
    Region as ScwRegion,
)
from scaleway_core.utils import (
    WaitForOptions,
    validate_path_param,
    fetch_all_pages,
    wait_for_resource,
)
from .types import (
    HostingStatus,
    ListHostingsRequestOrderBy,
    ListOffersRequestOrderBy,
    CheckUserOwnsDomainRequest,
    CheckUserOwnsDomainResponse,
    ClassicMailApiCreateMailboxRequest,
    ClassicMailApiUpdateMailboxRequest,
    ControlPanel,
    CreateHostingRequest,
    CreateHostingRequestDomainConfiguration,
    DnsRecords,
    EmailAddress,
    Hosting,
    ListControlPanelsResponse,
    ListHostingsResponse,
    ListMailboxesResponse,
    ListOffersResponse,
    Mailbox,
    ResetHostingPasswordResponse,
    Session,
    UpdateHostingRequest,
)
from .content import (
    HOSTING_TRANSIENT_STATUSES,
)
from .marshalling import (
    unmarshal_Hosting,
    unmarshal_Mailbox,
    unmarshal_CheckUserOwnsDomainResponse,
    unmarshal_DnsRecords,
    unmarshal_ListControlPanelsResponse,
    unmarshal_ListHostingsResponse,
    unmarshal_ListMailboxesResponse,
    unmarshal_ListOffersResponse,
    unmarshal_ResetHostingPasswordResponse,
    unmarshal_Session,
    marshal_CheckUserOwnsDomainRequest,
    marshal_ClassicMailApiCreateMailboxRequest,
    marshal_ClassicMailApiUpdateMailboxRequest,
    marshal_CreateHostingRequest,
    marshal_UpdateHostingRequest,
)
from ...std.types import (
    LanguageCode as StdLanguageCode,
)


class WebhostingV1Alpha1API(API):
    """
    This API allows you to manage your Web Hosting services.
    """

    def create_hosting(
        self,
        *,
        offer_id: str,
        domain: str,
        region: Optional[ScwRegion] = None,
        project_id: Optional[str] = None,
        email: Optional[str] = None,
        tags: Optional[List[str]] = None,
        option_ids: Optional[List[str]] = None,
        language: Optional[StdLanguageCode] = None,
        domain_configuration: Optional[CreateHostingRequestDomainConfiguration] = None,
    ) -> Hosting:
        """
        Order a Web Hosting plan.
        Order a Web Hosting plan, specifying the offer type required via the `offer_id` parameter.
        :param offer_id: ID of the selected offer for the Web Hosting plan.
        :param domain: Domain name to link to the Web Hosting plan. You must already own this domain name, and have completed the DNS validation process beforehand.
        :param region: Region to target. If none is passed will use default region from the config.
        :param project_id: ID of the Scaleway Project in which to create the Web Hosting plan.
        :param email: Contact email for the Web Hosting client.
        :param tags: List of tags for the Web Hosting plan.
        :param option_ids: IDs of any selected additional options for the Web Hosting plan.
        :param language: Default language for the control panel interface.
        :param domain_configuration: Indicates whether to update hosting domain name servers and DNS records for domains managed by Scaleway Elements.
        :return: :class:`Hosting <Hosting>`

        Usage:
        ::

            result = api.create_hosting(
                offer_id="example",
                domain="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "POST",
            f"/webhosting/v1alpha1/regions/{param_region}/hostings",
            body=marshal_CreateHostingRequest(
                CreateHostingRequest(
                    offer_id=offer_id,
                    domain=domain,
                    region=region,
                    project_id=project_id,
                    email=email,
                    tags=tags,
                    option_ids=option_ids,
                    language=language,
                    domain_configuration=domain_configuration,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Hosting(res.json())

    def list_hostings(
        self,
        *,
        region: Optional[ScwRegion] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListHostingsRequestOrderBy] = None,
        tags: Optional[List[str]] = None,
        statuses: Optional[List[HostingStatus]] = None,
        domain: Optional[str] = None,
        project_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        control_panels: Optional[List[str]] = None,
    ) -> ListHostingsResponse:
        """
        List all Web Hosting plans.
        List all of your existing Web Hosting plans. Various filters are available to limit the results, including filtering by domain, status, tag and Project ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: Page number to return, from the paginated results (must be a positive integer).
        :param page_size: Number of Web Hosting plans to return (must be a positive integer lower or equal to 100).
        :param order_by: Sort order for Web Hosting plans in the response.
        :param tags: Tags to filter for, only Web Hosting plans with matching tags will be returned.
        :param statuses: Statuses to filter for, only Web Hosting plans with matching statuses will be returned.
        :param domain: Domain to filter for, only Web Hosting plans associated with this domain will be returned.
        :param project_id: Project ID to filter for, only Web Hosting plans from this Project will be returned.
        :param organization_id: Organization ID to filter for, only Web Hosting plans from this Organization will be returned.
        :param control_panels: Name of the control panel to filter for, only Web Hosting plans from this control panel will be returned.
        :return: :class:`ListHostingsResponse <ListHostingsResponse>`

        Usage:
        ::

            result = api.list_hostings()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/webhosting/v1alpha1/regions/{param_region}/hostings",
            params={
                "control_panels": control_panels,
                "domain": domain,
                "order_by": order_by,
                "organization_id": organization_id
                or self.client.default_organization_id,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
                "project_id": project_id or self.client.default_project_id,
                "statuses": statuses,
                "tags": tags,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListHostingsResponse(res.json())

    def list_hostings_all(
        self,
        *,
        region: Optional[ScwRegion] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        order_by: Optional[ListHostingsRequestOrderBy] = None,
        tags: Optional[List[str]] = None,
        statuses: Optional[List[HostingStatus]] = None,
        domain: Optional[str] = None,
        project_id: Optional[str] = None,
        organization_id: Optional[str] = None,
        control_panels: Optional[List[str]] = None,
    ) -> List[Hosting]:
        """
        List all Web Hosting plans.
        List all of your existing Web Hosting plans. Various filters are available to limit the results, including filtering by domain, status, tag and Project ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: Page number to return, from the paginated results (must be a positive integer).
        :param page_size: Number of Web Hosting plans to return (must be a positive integer lower or equal to 100).
        :param order_by: Sort order for Web Hosting plans in the response.
        :param tags: Tags to filter for, only Web Hosting plans with matching tags will be returned.
        :param statuses: Statuses to filter for, only Web Hosting plans with matching statuses will be returned.
        :param domain: Domain to filter for, only Web Hosting plans associated with this domain will be returned.
        :param project_id: Project ID to filter for, only Web Hosting plans from this Project will be returned.
        :param organization_id: Organization ID to filter for, only Web Hosting plans from this Organization will be returned.
        :param control_panels: Name of the control panel to filter for, only Web Hosting plans from this control panel will be returned.
        :return: :class:`List[Hosting] <List[Hosting]>`

        Usage:
        ::

            result = api.list_hostings_all()
        """

        return fetch_all_pages(
            type=ListHostingsResponse,
            key="hostings",
            fetcher=self.list_hostings,
            args={
                "region": region,
                "page": page,
                "page_size": page_size,
                "order_by": order_by,
                "tags": tags,
                "statuses": statuses,
                "domain": domain,
                "project_id": project_id,
                "organization_id": organization_id,
                "control_panels": control_panels,
            },
        )

    def get_hosting(
        self,
        *,
        hosting_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Hosting:
        """
        Get a Web Hosting plan.
        Get the details of one of your existing Web Hosting plans, specified by its `hosting_id`.
        :param hosting_id: Hosting ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Hosting <Hosting>`

        Usage:
        ::

            result = api.get_hosting(
                hosting_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_hosting_id = validate_path_param("hosting_id", hosting_id)

        res = self._request(
            "GET",
            f"/webhosting/v1alpha1/regions/{param_region}/hostings/{param_hosting_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Hosting(res.json())

    def wait_for_hosting(
        self,
        *,
        hosting_id: str,
        region: Optional[ScwRegion] = None,
        options: Optional[WaitForOptions[Hosting, bool]] = None,
    ) -> Hosting:
        """
        Get a Web Hosting plan.
        Get the details of one of your existing Web Hosting plans, specified by its `hosting_id`.
        :param hosting_id: Hosting ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Hosting <Hosting>`

        Usage:
        ::

            result = api.get_hosting(
                hosting_id="example",
            )
        """

        if not options:
            options = WaitForOptions()

        if not options.stop:
            options.stop = lambda res: res.status not in HOSTING_TRANSIENT_STATUSES

        return wait_for_resource(
            fetcher=self.get_hosting,
            options=options,
            args={
                "hosting_id": hosting_id,
                "region": region,
            },
        )

    def update_hosting(
        self,
        *,
        hosting_id: str,
        region: Optional[ScwRegion] = None,
        email: Optional[str] = None,
        tags: Optional[List[str]] = None,
        option_ids: Optional[List[str]] = None,
        offer_id: Optional[str] = None,
        protected: Optional[bool] = None,
    ) -> Hosting:
        """
        Update a Web Hosting plan.
        Update the details of one of your existing Web Hosting plans, specified by its `hosting_id`. You can update parameters including the contact email address, tags, options and offer.
        :param hosting_id: Hosting ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :param email: New contact email for the Web Hosting plan.
        :param tags: New tags for the Web Hosting plan.
        :param option_ids: IDs of the new options for the Web Hosting plan.
        :param offer_id: ID of the new offer for the Web Hosting plan.
        :param protected: Whether the hosting is protected or not.
        :return: :class:`Hosting <Hosting>`

        Usage:
        ::

            result = api.update_hosting(
                hosting_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_hosting_id = validate_path_param("hosting_id", hosting_id)

        res = self._request(
            "PATCH",
            f"/webhosting/v1alpha1/regions/{param_region}/hostings/{param_hosting_id}",
            body=marshal_UpdateHostingRequest(
                UpdateHostingRequest(
                    hosting_id=hosting_id,
                    region=region,
                    email=email,
                    tags=tags,
                    option_ids=option_ids,
                    offer_id=offer_id,
                    protected=protected,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Hosting(res.json())

    def delete_hosting(
        self,
        *,
        hosting_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Hosting:
        """
        Delete a Web Hosting plan.
        Delete a Web Hosting plan, specified by its `hosting_id`. Note that deletion is not immediate: it will take place at the end of the calendar month, after which time your Web Hosting plan and all its data (files and emails) will be irreversibly lost.
        :param hosting_id: Hosting ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Hosting <Hosting>`

        Usage:
        ::

            result = api.delete_hosting(
                hosting_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_hosting_id = validate_path_param("hosting_id", hosting_id)

        res = self._request(
            "DELETE",
            f"/webhosting/v1alpha1/regions/{param_region}/hostings/{param_hosting_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Hosting(res.json())

    def restore_hosting(
        self,
        *,
        hosting_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Hosting:
        """
        Restore a Web Hosting plan.
        When you [delete a Web Hosting plan](#path-hostings-delete-a-hosting), definitive deletion does not take place until the end of the calendar month. In the time between initiating the deletion, and definitive deletion at the end of the month, you can choose to **restore** the Web Hosting plan, using this endpoint and specifying its `hosting_id`.
        :param hosting_id: Hosting ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Hosting <Hosting>`

        Usage:
        ::

            result = api.restore_hosting(
                hosting_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_hosting_id = validate_path_param("hosting_id", hosting_id)

        res = self._request(
            "POST",
            f"/webhosting/v1alpha1/regions/{param_region}/hostings/{param_hosting_id}/restore",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Hosting(res.json())

    def get_domain_dns_records(
        self,
        *,
        domain: str,
        region: Optional[ScwRegion] = None,
    ) -> DnsRecords:
        """
        Get DNS records.
        Get the set of DNS records of a specified domain associated with a Web Hosting plan.
        :param domain: Domain associated with the DNS records.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`DnsRecords <DnsRecords>`

        Usage:
        ::

            result = api.get_domain_dns_records(
                domain="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_domain = validate_path_param("domain", domain)

        res = self._request(
            "GET",
            f"/webhosting/v1alpha1/regions/{param_region}/domains/{param_domain}/dns-records",
        )

        self._throw_on_error(res)
        return unmarshal_DnsRecords(res.json())

    def check_user_owns_domain(
        self,
        *,
        domain: str,
        region: Optional[ScwRegion] = None,
        project_id: Optional[str] = None,
    ) -> CheckUserOwnsDomainResponse:
        """
        "Check whether you own this domain or not.".
        :param domain: Domain for which ownership is to be verified.
        :param region: Region to target. If none is passed will use default region from the config.
        :param project_id: ID of the project currently in use.
        :return: :class:`CheckUserOwnsDomainResponse <CheckUserOwnsDomainResponse>`

        Usage:
        ::

            result = api.check_user_owns_domain(
                domain="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_domain = validate_path_param("domain", domain)

        res = self._request(
            "POST",
            f"/webhosting/v1alpha1/regions/{param_region}/domains/{param_domain}/check-ownership",
            body=marshal_CheckUserOwnsDomainRequest(
                CheckUserOwnsDomainRequest(
                    domain=domain,
                    region=region,
                    project_id=project_id,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_CheckUserOwnsDomainResponse(res.json())

    def list_offers(
        self,
        *,
        without_options: bool,
        only_options: bool,
        region: Optional[ScwRegion] = None,
        order_by: Optional[ListOffersRequestOrderBy] = None,
        hosting_id: Optional[str] = None,
        control_panels: Optional[List[str]] = None,
    ) -> ListOffersResponse:
        """
        List all offers.
        List the different Web Hosting offers, and their options, available to order from Scaleway.
        :param without_options: Defines whether the response should consist of offers only, without options.
        :param only_options: Defines whether the response should consist of options only, without offers.
        :param region: Region to target. If none is passed will use default region from the config.
        :param order_by: Sort order of offers in the response.
        :param hosting_id: ID of a Web Hosting plan, to check compatibility with returned offers (in case of wanting to update the plan).
        :param control_panels: Name of the control panel to filter for.
        :return: :class:`ListOffersResponse <ListOffersResponse>`

        Usage:
        ::

            result = api.list_offers(
                without_options=False,
                only_options=False,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/webhosting/v1alpha1/regions/{param_region}/offers",
            params={
                "control_panels": control_panels,
                "hosting_id": hosting_id,
                "only_options": only_options,
                "order_by": order_by,
                "without_options": without_options,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListOffersResponse(res.json())

    def list_control_panels(
        self,
        *,
        region: Optional[ScwRegion] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> ListControlPanelsResponse:
        """
        "List the control panels type: cpanel or plesk.".
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: Page number to return, from the paginated results (must be a positive integer).
        :param page_size: Number of control panels to return (must be a positive integer lower or equal to 100).
        :return: :class:`ListControlPanelsResponse <ListControlPanelsResponse>`

        Usage:
        ::

            result = api.list_control_panels()
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )

        res = self._request(
            "GET",
            f"/webhosting/v1alpha1/regions/{param_region}/control-panels",
            params={
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListControlPanelsResponse(res.json())

    def list_control_panels_all(
        self,
        *,
        region: Optional[ScwRegion] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
    ) -> List[ControlPanel]:
        """
        "List the control panels type: cpanel or plesk.".
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: Page number to return, from the paginated results (must be a positive integer).
        :param page_size: Number of control panels to return (must be a positive integer lower or equal to 100).
        :return: :class:`List[ControlPanel] <List[ControlPanel]>`

        Usage:
        ::

            result = api.list_control_panels_all()
        """

        return fetch_all_pages(
            type=ListControlPanelsResponse,
            key="control_panels",
            fetcher=self.list_control_panels,
            args={
                "region": region,
                "page": page,
                "page_size": page_size,
            },
        )

    def create_session(
        self,
        *,
        hosting_id: str,
        region: Optional[ScwRegion] = None,
    ) -> Session:
        """
        Create a user session.
        :param hosting_id: Hosting ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Session <Session>`

        Usage:
        ::

            result = api.create_session(
                hosting_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_hosting_id = validate_path_param("hosting_id", hosting_id)

        res = self._request(
            "POST",
            f"/webhosting/v1alpha1/regions/{param_region}/hostings/{param_hosting_id}/sessions",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_Session(res.json())

    def reset_hosting_password(
        self,
        *,
        hosting_id: str,
        region: Optional[ScwRegion] = None,
    ) -> ResetHostingPasswordResponse:
        """
        :param hosting_id: UUID of the hosting.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`ResetHostingPasswordResponse <ResetHostingPasswordResponse>`

        Usage:
        ::

            result = api.reset_hosting_password(
                hosting_id="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_hosting_id = validate_path_param("hosting_id", hosting_id)

        res = self._request(
            "POST",
            f"/webhosting/v1alpha1/regions/{param_region}/hostings/{param_hosting_id}/reset-password",
            body={},
        )

        self._throw_on_error(res)
        return unmarshal_ResetHostingPasswordResponse(res.json())


class WebhostingV1Alpha1ClassicMailAPI(API):
    """
    This API allows you to manage your mailboxes for your Web Hosting services.
    """

    def create_mailbox(
        self,
        *,
        online_id: int,
        password: str,
        region: Optional[ScwRegion] = None,
        email: Optional[EmailAddress] = None,
    ) -> Mailbox:
        """
        Create a new mailbox within your hosting plan.
        :param online_id: The Online hosting ID.
        :param password: Password for the new mailbox.
        :param region: Region to target. If none is passed will use default region from the config.
        :param email: The email address of the mailbox.
        :return: :class:`Mailbox <Mailbox>`

        Usage:
        ::

            result = api.create_mailbox(
                online_id=1,
                password="example",
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_online_id = validate_path_param("online_id", online_id)

        res = self._request(
            "POST",
            f"/webhosting/v1alpha1/regions/{param_region}/classic-hostings/{param_online_id}/mailboxes",
            body=marshal_ClassicMailApiCreateMailboxRequest(
                ClassicMailApiCreateMailboxRequest(
                    online_id=online_id,
                    password=password,
                    region=region,
                    email=email,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Mailbox(res.json())

    def get_mailbox(
        self,
        *,
        online_id: int,
        mailbox_id: int,
        region: Optional[ScwRegion] = None,
    ) -> Mailbox:
        """
        Get a mailbox by id within your hosting plan.
        :param online_id: The Online hosting ID.
        :param mailbox_id: The ID of the mailbox to get.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Mailbox <Mailbox>`

        Usage:
        ::

            result = api.get_mailbox(
                online_id=1,
                mailbox_id=1,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_online_id = validate_path_param("online_id", online_id)
        param_mailbox_id = validate_path_param("mailbox_id", mailbox_id)

        res = self._request(
            "GET",
            f"/webhosting/v1alpha1/regions/{param_region}/classic-hostings/{param_online_id}/mailboxes/{param_mailbox_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Mailbox(res.json())

    def list_mailboxes(
        self,
        *,
        online_id: int,
        region: Optional[ScwRegion] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        domain: Optional[str] = None,
    ) -> ListMailboxesResponse:
        """
        List all mailboxes within your hosting plan.
        :param online_id: The Online hosting ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: Page number (must be a positive integer).
        :param page_size: Number of mailboxes to return (must be a positive integer lower or equal to 100).
        :param domain: Domain to filter the mailboxes.
        :return: :class:`ListMailboxesResponse <ListMailboxesResponse>`

        Usage:
        ::

            result = api.list_mailboxes(
                online_id=1,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_online_id = validate_path_param("online_id", online_id)

        res = self._request(
            "GET",
            f"/webhosting/v1alpha1/regions/{param_region}/classic-hostings/{param_online_id}/mailboxes",
            params={
                "domain": domain,
                "page": page,
                "page_size": page_size or self.client.default_page_size,
            },
        )

        self._throw_on_error(res)
        return unmarshal_ListMailboxesResponse(res.json())

    def list_mailboxes_all(
        self,
        *,
        online_id: int,
        region: Optional[ScwRegion] = None,
        page: Optional[int] = None,
        page_size: Optional[int] = None,
        domain: Optional[str] = None,
    ) -> List[Mailbox]:
        """
        List all mailboxes within your hosting plan.
        :param online_id: The Online hosting ID.
        :param region: Region to target. If none is passed will use default region from the config.
        :param page: Page number (must be a positive integer).
        :param page_size: Number of mailboxes to return (must be a positive integer lower or equal to 100).
        :param domain: Domain to filter the mailboxes.
        :return: :class:`List[Mailbox] <List[Mailbox]>`

        Usage:
        ::

            result = api.list_mailboxes_all(
                online_id=1,
            )
        """

        return fetch_all_pages(
            type=ListMailboxesResponse,
            key="mailboxes",
            fetcher=self.list_mailboxes,
            args={
                "online_id": online_id,
                "region": region,
                "page": page,
                "page_size": page_size,
                "domain": domain,
            },
        )

    def delete_mailbox(
        self,
        *,
        online_id: int,
        mailbox_id: int,
        region: Optional[ScwRegion] = None,
    ) -> Mailbox:
        """
        :param online_id: The Online hosting ID.
        :param mailbox_id: The ID of the mailbox to delete.
        :param region: Region to target. If none is passed will use default region from the config.
        :return: :class:`Mailbox <Mailbox>`

        Usage:
        ::

            result = api.delete_mailbox(
                online_id=1,
                mailbox_id=1,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_online_id = validate_path_param("online_id", online_id)
        param_mailbox_id = validate_path_param("mailbox_id", mailbox_id)

        res = self._request(
            "DELETE",
            f"/webhosting/v1alpha1/regions/{param_region}/classic-hostings/{param_online_id}/mailboxes/{param_mailbox_id}",
        )

        self._throw_on_error(res)
        return unmarshal_Mailbox(res.json())

    def update_mailbox(
        self,
        *,
        online_id: int,
        mailbox_id: int,
        region: Optional[ScwRegion] = None,
        password: Optional[str] = None,
    ) -> Mailbox:
        """
        Update the mailbox within your hosting plan.
        :param online_id: The Online hosting ID.
        :param mailbox_id: The ID of the mailbox to update.
        :param region: Region to target. If none is passed will use default region from the config.
        :param password: New password for the mailbox.
        :return: :class:`Mailbox <Mailbox>`

        Usage:
        ::

            result = api.update_mailbox(
                online_id=1,
                mailbox_id=1,
            )
        """

        param_region = validate_path_param(
            "region", region or self.client.default_region
        )
        param_online_id = validate_path_param("online_id", online_id)
        param_mailbox_id = validate_path_param("mailbox_id", mailbox_id)

        res = self._request(
            "PATCH",
            f"/webhosting/v1alpha1/regions/{param_region}/classic-hostings/{param_online_id}/mailboxes/{param_mailbox_id}",
            body=marshal_ClassicMailApiUpdateMailboxRequest(
                ClassicMailApiUpdateMailboxRequest(
                    online_id=online_id,
                    mailbox_id=mailbox_id,
                    region=region,
                    password=password,
                ),
                self.client,
            ),
        )

        self._throw_on_error(res)
        return unmarshal_Mailbox(res.json())



# This file was automatically generated. DO NOT EDIT.
# If you have any remark or suggestion do not hesitate to open an issue.
from .types import DNSStageType
from .types import LbOriginError
from .types import ListBackendStagesRequestOrderBy
from .types import ListCacheStagesRequestOrderBy
from .types import ListDNSStagesRequestOrderBy
from .types import ListPipelinesRequestOrderBy
from .types import ListPipelinesWithStagesRequestOrderBy
from .types import ListPurgeRequestsRequestOrderBy
from .types import ListTLSStagesRequestOrderBy
from .types import PipelineErrorCode
from .types import PipelineErrorSeverity
from .types import PipelineErrorStage
from .types import PipelineErrorType
from .types import PipelineStatus
from .content import PIPELINE_TRANSIENT_STATUSES
from .types import PlanName
from .types import PurgeRequestStatus
from .content import PURGE_REQUEST_TRANSIENT_STATUSES
from .types import ScalewayLb
from .types import ScalewayLbBackendConfig
from .types import ScalewayS3BackendConfig
from .types import PipelineError
from .types import TLSSecret
from .types import BackendStage
from .types import CacheStage
from .types import DNSStage
from .types import Pipeline
from .types import TLSStage
from .types import CheckPEMChainRequestSecretChain
from .types import PlanDetails
from .types import PlanUsageDetails
from .types import PipelineStages
from .types import PurgeRequest
from .types import TLSSecretsConfig
from .types import CheckDomainRequest
from .types import CheckDomainResponse
from .types import CheckLbOriginRequest
from .types import CheckLbOriginResponse
from .types import CheckPEMChainRequest
from .types import CheckPEMChainResponse
from .types import CreateBackendStageRequest
from .types import CreateCacheStageRequest
from .types import CreateDNSStageRequest
from .types import CreatePipelineRequest
from .types import CreatePurgeRequestRequest
from .types import CreateTLSStageRequest
from .types import DeleteBackendStageRequest
from .types import DeleteCacheStageRequest
from .types import DeleteCurrentPlanRequest
from .types import DeleteDNSStageRequest
from .types import DeletePipelineRequest
from .types import DeleteTLSStageRequest
from .types import GetBackendStageRequest
from .types import GetBillingRequest
from .types import GetBillingResponse
from .types import GetCacheStageRequest
from .types import GetCurrentPlanRequest
from .types import GetDNSStageRequest
from .types import GetPipelineRequest
from .types import GetPurgeRequestRequest
from .types import GetTLSStageRequest
from .types import ListBackendStagesRequest
from .types import ListBackendStagesResponse
from .types import ListCacheStagesRequest
from .types import ListCacheStagesResponse
from .types import ListDNSStagesRequest
from .types import ListDNSStagesResponse
from .types import ListPipelinesRequest
from .types import ListPipelinesResponse
from .types import ListPipelinesWithStagesRequest
from .types import ListPipelinesWithStagesResponse
from .types import ListPlansResponse
from .types import ListPurgeRequestsRequest
from .types import ListPurgeRequestsResponse
from .types import ListTLSStagesRequest
from .types import ListTLSStagesResponse
from .types import Plan
from .types import SelectPlanRequest
from .types import UpdateBackendStageRequest
from .types import UpdateCacheStageRequest
from .types import UpdateDNSStageRequest
from .types import UpdatePipelineRequest
from .types import UpdateTLSStageRequest
from .api import EdgeServicesV1Alpha1API

__all__ = [
    "DNSStageType",
    "LbOriginError",
    "ListBackendStagesRequestOrderBy",
    "ListCacheStagesRequestOrderBy",
    "ListDNSStagesRequestOrderBy",
    "ListPipelinesRequestOrderBy",
    "ListPipelinesWithStagesRequestOrderBy",
    "ListPurgeRequestsRequestOrderBy",
    "ListTLSStagesRequestOrderBy",
    "PipelineErrorCode",
    "PipelineErrorSeverity",
    "PipelineErrorStage",
    "PipelineErrorType",
    "PipelineStatus",
    "PIPELINE_TRANSIENT_STATUSES",
    "PlanName",
    "PurgeRequestStatus",
    "PURGE_REQUEST_TRANSIENT_STATUSES",
    "ScalewayLb",
    "ScalewayLbBackendConfig",
    "ScalewayS3BackendConfig",
    "PipelineError",
    "TLSSecret",
    "BackendStage",
    "CacheStage",
    "DNSStage",
    "Pipeline",
    "TLSStage",
    "CheckPEMChainRequestSecretChain",
    "PlanDetails",
    "PlanUsageDetails",
    "PipelineStages",
    "PurgeRequest",
    "TLSSecretsConfig",
    "CheckDomainRequest",
    "CheckDomainResponse",
    "CheckLbOriginRequest",
    "CheckLbOriginResponse",
    "CheckPEMChainRequest",
    "CheckPEMChainResponse",
    "CreateBackendStageRequest",
    "CreateCacheStageRequest",
    "CreateDNSStageRequest",
    "CreatePipelineRequest",
    "CreatePurgeRequestRequest",
    "CreateTLSStageRequest",
    "DeleteBackendStageRequest",
    "DeleteCacheStageRequest",
    "DeleteCurrentPlanRequest",
    "DeleteDNSStageRequest",
    "DeletePipelineRequest",
    "DeleteTLSStageRequest",
    "GetBackendStageRequest",
    "GetBillingRequest",
    "GetBillingResponse",
    "GetCacheStageRequest",
    "GetCurrentPlanRequest",
    "GetDNSStageRequest",
    "GetPipelineRequest",
    "GetPurgeRequestRequest",
    "GetTLSStageRequest",
    "ListBackendStagesRequest",
    "ListBackendStagesResponse",
    "ListCacheStagesRequest",
    "ListCacheStagesResponse",
    "ListDNSStagesRequest",
    "ListDNSStagesResponse",
    "ListPipelinesRequest",
    "ListPipelinesResponse",
    "ListPipelinesWithStagesRequest",
    "ListPipelinesWithStagesResponse",
    "ListPlansResponse",
    "ListPurgeRequestsRequest",
    "ListPurgeRequestsResponse",
    "ListTLSStagesRequest",
    "ListTLSStagesResponse",
    "Plan",
    "SelectPlanRequest",
    "UpdateBackendStageRequest",
    "UpdateCacheStageRequest",
    "UpdateDNSStageRequest",
    "UpdatePipelineRequest",
    "UpdateTLSStageRequest",
    "EdgeServicesV1Alpha1API",
]
